
class FloatTable {
  int rowCount;
  int columnCount;
  float[][] data;
  String[] rowNames;
  String[] columnNames;


  FloatTable(String filename) {
    String[] rows = loadStrings(filename);
    String[] columns = split(rows[0], TAB);// enregistre les noms de colonnes
    columnNames = subset(columns, 1); // ignore la cellule en haut à gauche
    columnCount = columnNames.length;

    rowNames = new String[rows.length-1];
    data = new float[rows.length-1][];

    //commence avec la 2eme ligne, 1er sont les noms
    for (int i = 1; i < rows.length; i++) {
      if (trim(rows[i]).length() == 0) {
        continue; // saute les lignes vides
      }

      // coupe les lignes aux tabs
      String[] cellules = split(rows[i], TAB);


      // copie les titres
      rowNames[rowCount] = cellules[0];
      // copie les données dans la table
      data[rowCount] = parseFloat(subset(cellules, 1));

      // increment les nombres des colonnes valide
      rowCount++;      
    }
    // reduit la liste des données si necessaire
    data = (float[][]) subset(data, 0, rowCount);
  }


  int getRowCount() {
    return rowCount;
  }

  String getRowName(int rowIndex) {
    return rowNames[rowIndex];
  } 

  String[] getRowNames() {
    return rowNames;
  }

  int getRowIndex(String name) {
    for (int i = 0; i < rowCount; i++) {
      if (rowNames[i].equals(name)) {
        return i;
      }
    }
    return -1;
  }


  int getColumnCount() {
    return columnCount;
  }


  String getColumnName(int colIndex) {
    return columnNames[colIndex];
  }


  String[] getColumnNames() {
    return columnNames;
  }


  float getFloat(int rowIndex, int col) {
    return data[rowIndex][col];
  }


  boolean isValid(int row, int col) {
    if (row < 0) return false;
    if (row >= rowCount) return false;
    if (col >= data[row].length) return false;
    if (col < 0) return false;
    return !Float.isNaN(data[row][col]);
  }

  float getColumnMax(int col) {
    float m = MIN_FLOAT;
    for (int row = 0; row < rowCount; row++) {
      if (isValid(row, col)) {
        if (data[row][col] > m) {
          m = data[row][col];
        }
      }
    }
    return m;
  }


  float getTableMin() {
    float m = MAX_FLOAT;
    for (int row = 0; row < rowCount; row++) {
      for (int col = 0; col < columnCount; col++) {
        if (isValid(row, col)) {
          if (data[row][col] < m) {
            m = data[row][col];
          }
        }
      }
    }
    return m;
  }


  float getTableMax() {
    float m = MIN_FLOAT;
    for (int row = 0; row < rowCount; row++) {
      for (int col = 0; col < columnCount; col++) {
        if (isValid(row, col)) {
          if (data[row][col] > m) {
            m = data[row][col];
          }
        }
      }
    }
    return m;
  }
}

